@@ -45,8 +45,10 @@ struct _CcKeyboardPanel
 
   /* Search */
   GtkWidget          *empty_search_placeholder;
+  GtkWidget          *search_bar;
   GtkWidget          *search_button;
   GtkWidget          *search_entry;
+  guint               search_bar_handler_id;
 
   /* Shortcuts */
   GtkWidget          *listbox;
@@ -142,6 +144,70 @@ shortcut_modified_changed_cb (CcKeyboardItem *item,
   gtk_widget_set_child_visible (button, !cc_keyboard_item_is_value_default (item));
 }
 
+static void
+reset_all_shortcuts_cb (GtkWidget *widget,
+                        gpointer   user_data)
+{
+  CcKeyboardPanel *self;
+  RowData *data;
+
+  self = user_data;
+
+  if (widget == (GtkWidget *) self->add_shortcut_row)
+    return;
+
+  data = g_object_get_data (G_OBJECT (widget), "data");
+
+  /* Don't reset custom shortcuts */
+  if (data->item->type == CC_KEYBOARD_ITEM_TYPE_GSETTINGS_PATH)
+    return;
+
+  /* cc_keyboard_manager_reset_shortcut() already resets conflicting shortcuts,
+   * so no other check is needed here. */
+  cc_keyboard_manager_reset_shortcut (self->manager, data->item);
+}
+
+static void
+reset_all_clicked_cb (CcKeyboardPanel *self)
+{
+  GtkWidget *dialog, *toplevel, *button;
+  guint response;
+
+  toplevel = gtk_widget_get_toplevel (GTK_WIDGET (self));
+  dialog = gtk_message_dialog_new (GTK_WINDOW (toplevel),
+                                   GTK_DIALOG_MODAL | GTK_DIALOG_USE_HEADER_BAR | GTK_DIALOG_DESTROY_WITH_PARENT,
+                                   GTK_MESSAGE_WARNING,
+                                   GTK_BUTTONS_NONE,
+                                   _("Reset All Shortcuts?"));
+
+  gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
+                                            _("Resetting the shortcuts may affect your custom shortcuts. "
+                                              "This cannot be undone."));
+
+  gtk_dialog_add_buttons (GTK_DIALOG (dialog),
+                          _("Cancel"), GTK_RESPONSE_CANCEL,
+                          _("Reset All"), GTK_RESPONSE_ACCEPT,
+                          NULL);
+
+  gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_CANCEL);
+
+  /* Make the "Reset All" button destructive */
+  button = gtk_dialog_get_widget_for_response (GTK_DIALOG (dialog), GTK_RESPONSE_ACCEPT);
+  gtk_style_context_add_class (gtk_widget_get_style_context (button), "destructive-action");
+
+  /* Reset shortcuts if accepted */
+  response = gtk_dialog_run (GTK_DIALOG (dialog));
+
+  if (response == GTK_RESPONSE_ACCEPT)
+    {
+      gtk_container_foreach (GTK_CONTAINER (self->listbox),
+                             reset_all_shortcuts_cb,
+                             self);
+    }
+
+  gtk_widget_destroy (dialog);
+}
+
 static void
 reset_shortcut_cb (GtkWidget      *reset_button,
                    CcKeyboardItem *item)
@@ -451,12 +517,19 @@ static void
 cc_keyboard_panel_finalize (GObject *object)
 {
   CcKeyboardPanel *self = CC_KEYBOARD_PANEL (object);
+  GtkWidget *window;
 
   g_clear_pointer (&self->pictures_regex, g_regex_unref);
   g_clear_object (&self->accelerator_sizegroup);
 
   cc_keyboard_option_clear_all ();
 
+  if (self->search_bar_handler_id != 0)
+    {
+      window = cc_shell_get_toplevel (cc_panel_get_shell (CC_PANEL (self)));
+      g_signal_handler_disconnect (window, self->search_bar_handler_id);
+    }
+
   G_OBJECT_CLASS (cc_keyboard_panel_parent_class)->finalize (object);
 }
 
@@ -473,6 +546,12 @@ cc_keyboard_panel_constructed (GObject *object)
   gtk_window_set_transient_for (GTK_WINDOW (self->shortcut_editor), toplevel);
 
   cc_shell_embed_widget_in_header (cc_panel_get_shell (CC_PANEL (self)), self->search_button);
+
+  self->search_bar_handler_id =
+    g_signal_connect_swapped (toplevel,
+                              "key-press-event",
+                              G_CALLBACK (gtk_search_bar_handle_event),
+                              self->search_bar);
 }
 
 static void
@@ -495,9 +574,11 @@ cc_keyboard_panel_class_init (CcKeyboardPanelClass *klass)
   gtk_widget_class_bind_template_child (widget_class, CcKeyboardPanel, add_shortcut_row);
   gtk_widget_class_bind_template_child (widget_class, CcKeyboardPanel, empty_search_placeholder);
   gtk_widget_class_bind_template_child (widget_class, CcKeyboardPanel, listbox);
+  gtk_widget_class_bind_template_child (widget_class, CcKeyboardPanel, search_bar);
   gtk_widget_class_bind_template_child (widget_class, CcKeyboardPanel, search_button);
   gtk_widget_class_bind_template_child (widget_class, CcKeyboardPanel, search_entry);
 
+  gtk_widget_class_bind_template_callback (widget_class, reset_all_clicked_cb);
   gtk_widget_class_bind_template_callback (widget_class, shortcut_row_activated);
 }
 
