@@ -24,8 +24,9 @@
 
 #include <glib-object.h>
 #include <gtk/gtk.h>
-#include <nm-client.h>
-#include <nm-remote-settings.h>
+#include <NetworkManager.h>
+
+#include "cc-network-panel.h"
 
 G_BEGIN_DECLS
 
@@ -56,6 +57,7 @@ struct _NetObjectClass
                                                         GtkSizeGroup    *heading_size_group);
         void                     (*delete)              (NetObject       *object);
         void                     (*refresh)             (NetObject       *object);
+        void                     (*edit)                (NetObject       *object);
 
         /* signal */
         void                     (* changed)            (NetObject      *object);
@@ -70,12 +72,13 @@ const gchar     *net_object_get_title                   (NetObject      *object)
 void             net_object_set_title                   (NetObject      *object,
                                                          const gchar    *title);
 NMClient        *net_object_get_client                  (NetObject      *object);
-NMRemoteSettings *net_object_get_remote_settings        (NetObject      *object);
 GCancellable    *net_object_get_cancellable             (NetObject      *object);
+CcNetworkPanel  *net_object_get_panel                   (NetObject      *object);
 void             net_object_emit_changed                (NetObject      *object);
 void             net_object_emit_removed                (NetObject      *object);
 void             net_object_delete                      (NetObject      *object);
 void             net_object_refresh                     (NetObject      *object);
+void             net_object_edit                        (NetObject      *object);
 GtkWidget       *net_object_add_to_notebook             (NetObject      *object,
                                                          GtkNotebook    *notebook,
                                                          GtkSizeGroup   *heading_size_group);
